/*****************************************************************************
**
**  mom_*.c
**
**  Description:
**     Contains Unigraphics entry points for the application.
**
*****************************************************************************/

#define _CRT_SECURE_NO_DEPRECATE 1

/*  Include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ERRNO.H>
#include <process.h>

#include <windows.h>
#include <shellapi.h>

#include <uf.h>
#include <uf_ui.h>
#include <uf_mom.h>

#include <uf_attr.h>
#include <uf_obj.h>
#include <uf_param.h>
#include <uf_param_indices.h>

#include <uf_ui_ont.h>

#include <uf_cam.h>
#include <uf_setup.h>
#include <uf_ncgroup.h>
#include <uf_oper.h>

#include <uf_ugopenint.h>

#include <uf_part.h>
#include <uf_assem.h>
#include <uf_ugmgr.h>

#include <uf_csys.h>

#include "mom_extend.hpp"

/****************************************************************************/

#define UF_CALL(X) (report( __FILE__, __LINE__, #X, (X)))

static int report( char *file, int line, char *call, int irc)
{
  if (irc)
  {
     char    messg[133];
     printf("%s, line %d:  %s\n", file, line, call);
     (UF_get_fail_message(irc, messg)) ?
       printf("    returned a %d\n", irc) :
       printf("    returned error %d:  %s\n", irc, messg);
  }
  return(irc);
}

void report_double_array(char *name, double *array, int rows, int cols)
{
    int  ii,  jj,  kk = 0;

    printf("%s\n", name);
    for (ii = 0; ii < rows; ii++)
    {
        printf("\t");
        for (jj = 0; jj < cols; jj++)
        {
            printf("%f", array[kk++]);
            if (jj < cols - 1) printf(", ");
        }
        printf("\n");
    }
}

/* */
/*----------------------------------------------------------------------------*/
static logical cycleGrp( tag_t   tag, void   *data )
{
   logical  is_group ;
   //char     name[UF_OPER_MAX_NAME_LEN + 1];
   int      ecode;
   tag_t    * mcs = (tag_t *)data;
   logical  ans ;
   int      ind = -1;

   ecode = UF_NCGROUP_is_group( tag, &is_group );
   ////if( is_group != TRUE ) return( FALSE );

   //name[0]='\0';
   //ecode = UF_OBJ_ask_name(tag, name);//   
   //printf("\n %s",name);

   ecode = UF_NCGROUP_is_a_member(tag,*mcs,&ans,&ind);
   //printf("\t ans=%d  ind=%d",(int)ans,ind);
   if ((int)ans) {
     *mcs = tag ;
     return( FALSE );
   }

   return( TRUE );
}




//===========================================================================
int do_it( char *param )
{
    int errorCode ;
    UF_MOM_id_t mom ;
    void *interp = NULL ;

    /* Get the TCL interpreter id from the ufusr param */
    errorCode=UF_CALL( UF_MOM_ask_interp_from_param ( param, &interp ) );
    if ( 0 != errorCode ) {
      UF_print_syslog("Don't get the TCL interpreter id from the ufusr 'param'\nDLL terminated...\n",FALSE);
      return(-1) ;
    }

    /* Get the MOM id from the ufusr param */
    errorCode=UF_CALL( UF_MOM_ask_mom (param, &mom) ) ;
    if ( 0 != errorCode ) {
      UF_print_syslog("Don't get the MOM id from the ufusr param\nDLL terminated...\n",FALSE);
      return(-1) ;
    }

   /*  */
   UF_print_syslog("- >pb_base_stub start \n",FALSE);

   /*   */

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_ui_set_status", MOM_ui_set_status ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_ui_set_status \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_box_message", mom_box_message ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_box_message \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_ui_message", mom_UI_message_dialog ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_ui_message \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_abort_ex", mom_UI_message_dialog_abort ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_abort_ex \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_info_message", mom_UI_display_nonmodal_msg ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_info_message \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_ui_filebox", mom_UI_create_filebox ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_ui_filebox \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_assign_attribute", mom_create_attribute ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_set_attribute_program \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_set_attribute_program", mom_set_attribute_program ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_set_attribute_program \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_ask_part_attr", mom_ask_part_attr ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_ask_part_attr \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_call_grip", mom_call_grip ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_call_grip \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_uf_param_ask_value", mom_uf_param_ask_value ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_uf_param_ask_value \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_get_ini_string", mom_IniRead ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_get_ini_string \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_remove_file_ex", mom_remove_ex ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_remove_file_ex \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_deletefile", mom_deletefile ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_deletefile \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_createfile", mom_createfile ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_createfile \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_run_exec", mom_RunExecute ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_run_exec \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_shell_exec", mom_ShellExecute ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_shell_exec \n",FALSE);

   errorCode = UF_MOM_extend_xlator ( mom , "MOM_about_stub", mom_AboutStub ) ;
   if ( 0 != errorCode ) UF_print_syslog("Don't get UF_MOM_extend_xlator ... MOM_about_stub \n",FALSE);

   UF_print_syslog("- >pb_base_stub end \n",FALSE);

 return (0);
}



/*-----------------------------------------------------------------------
 * Function Name: MOM_ui_set_status
 *
 *   -   
   MOM_ui_set_status ?str

 -----------------------------------------------------------------------*/
static int MOM_ui_set_status
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  char message[133]="";
  if (argc>=2) sprintf(message,"%.132s",argv[1]);
  /* Print out the message */
  UF_UI_set_status( message );
  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function Name: mom_box_message
 *
 *   - 
   MOM_box_message ?m1 ?m2 ?m3 ?m4 ?m5 ....

          (  uc1601)   
       :

         m =m1 \n m2 \n m3 \n m4 \n m5 \n ...

 -----------------------------------------------------------------------*/
static int mom_box_message
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  int   i , l ;
  char  str[132];

  /*   */
  l=0;
  str[0]='\0';
  for (i=0;i<argc;i++) {
    if (i>=1) {
      l+=(int)(strlen(argv[i])+1);
      if (l>=132 ) break ;
      //strncat(str,argv[i], strlen(argv[i]));
      strcat(str,argv[i]); strcat(str,"\n");
    }
  }

  if (argc>=2) uc1601 (str, TRUE );

  intrp->result="0";
  intrp->errorLine=TCL_OK;

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function Name: mom_UI_message_dialog
 *
 *   - 
   set ret [ MOM_ui_message ?typesign ?button ?m1 ?m2 ?m3 ?m4 ?m5 .... ]

     typesign =
      { ERROR|WARNING|INFORMATION|QUESTION } -   ()
      { ERROR|WARN|INFO|? } -  
     button =
      ABORTRETRYIGNORE  The message box contains three push buttons: Abort, Retry, and Ignore.
      OK                The message box contains one push button: OK. This is the default.
      OKCANCEL          The message box contains two push buttons: OK and Cancel.
      RETRYCANCEL       The message box contains two push buttons: Retry and Cancel.
      YESNO             The message box contains two push buttons: Yes and No.
      YESNOCANCEL       The message box contains three push buttons: Yes, No, and Cancel.

    
    ret = {1 2 3} if message box contains three push buttons
    ret = {1 3}   if message box contains two push buttons
 -----------------------------------------------------------------------*/
static int mom_UI_message_dialog
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  UF_MOM_id_t     mom_id = ( UF_MOM_id_t ) client_data ;
  char str_return[10];
  const int Size = 30 ;
  int  i , l , cnt;
  UF_UI_MESSAGE_DIALOG_TYPE type_message ;
  char *mes1[Size] ;
  UF_UI_message_buttons_t buttons0 ;
  UF_UI_message_buttons_t buttons1 = { TRUE, TRUE, TRUE, "Abort", "Retry", "Ignore", 1, 2, 3 };
  UF_UI_message_buttons_t buttons2 = { TRUE, FALSE, FALSE, "OK", NULL, NULL, 1, 0, 0 };
  UF_UI_message_buttons_t buttons3 = { TRUE, FALSE, TRUE, "OK", NULL, "Cancel", 1, 0, 3 };
  UF_UI_message_buttons_t buttons4 = { TRUE, FALSE, TRUE, "Retry", NULL, "Cancel", 1, 0, 3 };
  UF_UI_message_buttons_t buttons5 = { TRUE, FALSE, TRUE, "Yes", NULL, "No", 1, 0, 3 };
  UF_UI_message_buttons_t buttons6 = { TRUE, TRUE, TRUE, "Yes", "No", "Cancel", 1, 2, 3 };
  int  response ;

  type_message=UF_UI_MESSAGE_INFORMATION ;
  /*UF_UI_MESSAGE_ERROR  UF_UI_MESSAGE_WARNING  UF_UI_MESSAGE_INFORMATION  UF_UI_MESSAGE_QUESTION*/
  buttons0 = buttons2;

  /*   */
  cnt=0;
  for (i=0;i<argc;i++) {
   if (i==1 || i==2) {
      //
      if (0==strcmp(argv[i],"ERROR")) { type_message=UF_UI_MESSAGE_ERROR ; continue ; }
      if ((0==strcmp(argv[i],"WARNING"))||(0==strcmp(argv[i],"WARN"))) { type_message=UF_UI_MESSAGE_WARNING ; continue ; }
      if ((0==strcmp(argv[i],"INFORMATION"))||(0==strcmp(argv[i],"INFO"))) { type_message=UF_UI_MESSAGE_INFORMATION ; continue ; }
      if ((0==strcmp(argv[i],"QUESTION"))||(0==strcmp(argv[i],"?"))) { type_message=UF_UI_MESSAGE_QUESTION ; continue ; }
      //
      if (0==strcmp(argv[i],"ABORTRETRYIGNORE")) { buttons0=buttons1 ; continue ; }
      if (0==strcmp(argv[i],"OK")) { buttons0=buttons2  ; continue ; }
      if (0==strcmp(argv[i],"OKCANCEL")) { buttons0=buttons3  ; continue ; }
      if (0==strcmp(argv[i],"RETRYCANCEL")) { buttons0=buttons4  ; continue ; }
      if (0==strcmp(argv[i],"YESNO")) { buttons0=buttons5 ; continue ; }
      if (0==strcmp(argv[i],"YESNOCANCEL")) { buttons0=buttons6  ; continue ; }
      //
   }
   if (i>=1) {
     if (cnt>=Size) break ;
     l=(int)strlen(argv[i])+1;
     mes1[cnt]=new char [l];
     mes1[cnt][0]='\0'; sprintf(mes1[cnt],"%s",argv[i]);
     cnt++;
   }
  }

 response=0;
 if (argc>=2 && cnt!=0) {
    UF_UI_message_dialog("PostProcessor.....", type_message , mes1, cnt, TRUE, &buttons0, &response);

    for (i=0;i<cnt;i++) delete mes1[i] ;
    //delete [] mes1 ;
 }

 UF_MOM_set_int ( mom_id , "ui_message_return",response ) ;

 str_return[0]='\0'; sprintf(str_return,"%d",response);
 intrp->result=str_return;
 intrp->errorLine=TCL_OK;

 return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function Name: mom_UI_message_dialog_abort
 *    mom_UI_message_dialog
 *   - 
       .

   MOM_abort_ex ?m1 ?m2 ?m3 ?m4 ?m5 ....

      MOM_ui_message
   
     typesign = ERROR
     button   = ABORT

       (  UF_UI_message_dialog)
     
           .
 -----------------------------------------------------------------------*/
static int mom_UI_message_dialog_abort
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
 Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
 int   lb ;
 char *meslb ;
 const int Size = 15 ;
 int  i , l , cnt;
 char *mes1[Size] ;
 UF_UI_message_buttons_t buttons0 = { TRUE, FALSE, FALSE, "Abort", NULL, NULL, 1, 0, 0 }; ;
 int  response ;

 /*   */
 for (i=0,cnt=0,lb=0;i<argc;i++) {
  if (i>=1) {
    if (cnt>=Size) break ;
    l=(int)strlen(argv[i])+1;
    lb+=l;
    mes1[cnt]=new char [l];
    mes1[cnt][0]='\0'; sprintf(mes1[cnt],"%s",argv[i]);
    cnt++;
  }
 }
 if (cnt) {
  meslb=new char [lb];
  meslb[0]='\0';
  for (i=1;i<argc;i++) { strcat(meslb,argv[i]);   strcat(meslb,"\n"); }
  strcat(meslb,"\0");
 }

 if (argc>=2 && cnt!=0) {
    response=0;
    UF_UI_message_dialog("PostProcessor Abort..", UF_UI_MESSAGE_ERROR , mes1, cnt, TRUE, &buttons0, &response);
    for (i=0;i<cnt;i++) delete mes1[i] ;
    //delete [] mes1 ;
 }

 intrp->result=meslb;
 intrp->errorLine=TCL_ERROR ;

 if (cnt) delete meslb ;

 return ( TCL_ERROR ) ;
}


/*-----------------------------------------------------------------------
 * Function Name: mom_info_message
 *
 *   -  
 MOM_info_message ?where ?m1 ?m2 ?m3 ?m4 ?m5 ....

    where
      { CURSOR|CASCADE|RIGHT|LEFT } -   

       (  UF_UI_display_nonmodal_msg),
     .
       :

         m =m1 \n m2 \n m3 \n m4 \n m5 \n ...
 -----------------------------------------------------------------------*/
static int mom_UI_display_nonmodal_msg
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  const int Size = 1024 ;
  int   i , l;
  char  str[Size] ;
  int  posMethod ;

/*
UF_UI_MSG_POS_CURSOR = the dialog's OK button will be under the mouse cursor.
UF_UI_MSG_POS_CASCADE = the dialog will be positioned diagonally accross the screen relative to other non-modal message dialogs.
UF_UI_MSG_POS_BOTTOM_RIGHT = the dialog will be positioned at the bottom right corner of the graphics window.
UF_UI_MSG_POS_TOP_LEFT = the dialog will be positioned at the top left corner of the screen.
*/
  posMethod=UF_UI_MSG_POS_TOP_LEFT ;
  /*   */
  l=0;
  str[0]='\0';
  for (i=0;i<argc;i++) {
   if (i==1) {
      if (0==strcmp(argv[i],"CURSOR")) { posMethod=UF_UI_MSG_POS_CURSOR ; continue ; }
      if (0==strcmp(argv[i],"CASCADE")) { posMethod=UF_UI_MSG_POS_CASCADE ; continue ; }
      if (0==strcmp(argv[i],"RIGHT")) { posMethod=UF_UI_MSG_POS_BOTTOM_RIGHT ; continue ; }
      if (0==strcmp(argv[i],"LEFT")) { posMethod=UF_UI_MSG_POS_TOP_LEFT ; continue ; }
   }
   if (i>=1) {
     l+=((int)strlen(argv[i])+1);
     if (l>=Size) break ;
     strcat(str,argv[i]); strcat(str,"\n");
   }
  }

  if (argc>=2) {
    UF_UI_display_nonmodal_msg( "PostProcessor.....",str, posMethod );
  }

  intrp->result="";
  intrp->errorLine=TCL_OK ;

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function Name: mom_UI_create_filebox
 * 
 *   .
  set file [ MOM_ui_filebox ?file ?ext ]

  :
     file                    -    
     mom_ui_filebox_return   -    (,cancel),   .
     mom_ui_filebox_filename -  

 -----------------------------------------------------------------------*/
static int mom_UI_create_filebox
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  UF_MOM_id_t mom_id = ( UF_MOM_id_t ) client_data ;
  char filter_string[132] = "*";
  char filename[132]="";
  int  response=-1;

  /*   */
  if (argc>=2) sprintf(filename,"%s",argv[1]);
  if (argc>=3) sprintf(filter_string,"%s",argv[2]);

  UF_UI_create_filebox ("->Select File...","..Select File...",
                         filter_string, filename, filename, &response);

  UF_MOM_set_int ( mom_id , "ui_filebox_return",response ) ;
  UF_MOM_set_string ( mom_id , "ui_filebox_filename",filename ) ;

  intrp->result=filename;
  intrp->errorLine=TCL_OK;

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function :    
 *  MOM  {title}  {value|""|NULL|TIME}  {OPER|PROGRAM}

   set ret [ MOM_assign_attribute $title {$value|NULL|TIME} {operation_name|group_name} ]

      title                 -  
      value                 -  ,    UF_ATTR_string
        NULL                -    UF_ATTR_null
        TIME                -    UF_ATTR_time   -   
     {operation_name
      group_name}           -     (),   
     {OPER                  -      ( -  )
      PROGRAM}              -     ( )

    :
   ret=
     0 -  .
     1 -    .

 -----------------------------------------------------------------------*/
static int mom_create_attribute
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
 Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
/*  Variable Declarations */
    tag_t   oper = NULL_TAG;
    tag_t  group = NULL_TAG;
    UF_ATTR_value_t value;
    char     title[UF_ATTR_MAX_TITLE_LEN+1]="";
    char     string_value[UF_ATTR_MAX_STRING_LEN+1]="";
    int  err;
    //char * op_name ;
    const char * operation_name ;
    char oper_name[UF_OPER_MAX_NAME_LEN + 1]="";
    char prg_value[UF_OPER_MAX_NAME_LEN + 1]="";

    UF_MOM_id_t     mom_id = ( UF_MOM_id_t ) client_data ;
    UF_OPER_id_t    oper_id = ( UF_OPER_id_t ) client_data ;
//  UF_PATH_id_t    path_id = ( UF_PATH_id_t ) client_data ;
//  UF_CAM_exit_id_t exit_id= ( UF_CAM_exit_id_t ) client_data ;

  intrp->result="1";
  intrp->errorLine=TCL_OK;

  if (argc<=1) {
    UF_print_syslog(" mom_create_attribute - wrong number arguments\n",FALSE);
    return(TCL_OK);
  }

  title[0]='\0';
  string_value[0]='\0';
  oper_name[0]='\0';
  prg_value[0]='\0'; sprintf(prg_value,"%s","OPER" );

  /*   */
  if (argc>=2) { sprintf(title,"%s", argv[1]); }
  if (argc>=3) { sprintf(string_value,"%s", argv[2]); }
  if (argc>=4) { sprintf(oper_name,"%.30s",argv[3]); }
  if (argc>=5) { sprintf(prg_value,"%.30s", argv[4]); }

 /***************************************************************************/

  //err=UF_CALL( UF_OPER_ask_name( oper_id , oper_name ) );
  //if ( err ) { return (TCL_OK); }

  if ( oper_name[0]=='\0' ) {
    err = UF_MOM_ask_string ( mom_id , "operation_name" , &operation_name ) ;
    if ( err ) {
     UF_print_syslog(" mom_create_attribute - wrong 'operation_name'\n",FALSE);
     return (TCL_OK);
    }
    sprintf(oper_name,"%.30s",operation_name);
  }

  //printf("operation_name=%s",oper_name);

  //     
  err=UF_CALL( UF_OBJ_cycle_by_name(oper_name,&oper) );
  if (err) {
    UF_print_syslog(" mom_create_attribute - wrong tag for 'operation_name'\n",FALSE);
    return (TCL_OK);
  }

/*
  prt_tag = UF_PART_ask_display_part()
  if  (prt_tag != null_tag) {
    UF_OBJ_cycle_by_name_and_type(prt_tag,operation_name,100,TRUE,&oper );
  }
*/

  if (strcmp(prg_value,"PROGRAM") == 0) {
     UF_CALL( UF_OPER_ask_program_group ( oper, &group) );
     if (group!=null_tag) { oper=group; }
  }

  if (oper==NULL_TAG) {
     UF_print_syslog(" mom_create_attribute - wrong 'operation_name' has NULL_TAG\n",FALSE) ;
     return (TCL_OK);
  }

//  

/*          UF_ATTR_integer
            UF_ATTR_real,
            UF_ATTR_time
            UF_ATTR_null,
            UF_ATTR_string
            UF_ATTR_any
            UF_ATTR_reference  */

  if (strcmp(string_value,"NULL") == 0) {
/*  Assign a null attribute to the  */
    value.type = UF_ATTR_null;
    UF_CALL(UF_ATTR_assign(oper, title, value));
  } else if (strcmp(string_value,"TIME") == 0) {
         /*  Assign a date and time attribute to the  */
             value.type = UF_ATTR_time;
         /* Init to current date and time */
             UF_CALL(uc4583("", "", value.value.time));
             UF_CALL(UF_ATTR_assign(oper, title, value));
  } else {
    /*  Assign a string attribute to the object */
        value.type = UF_ATTR_string;
        value.value.string = string_value;
        UF_CALL(UF_ATTR_assign(oper, title , value));
  }

  intrp->result="0";
  intrp->errorLine=TCL_OK;

  return (TCL_OK);
}


/*-----------------------------------------------------------------------
 * Function :      
 *   MOM group_name file_name time machine

     :
      OPU_FILE_UP_NAME      -  
      OPU_FILE_UP_CREATE    -   
      OPU_FILE_UP_TIME      -  
      OPU_FILE_UP_MACHINE   - 

  1)    -     
      MOM_set_attribute_program
  2)    -
      MOM_set_attribute_program $mom_group_name $ptp_file_name $mom_machine_time $mom_machine_name
     
      MOM_set_attribute_program $mom_operation_name $ptp_file_name $mom_machine_time $mom_machine_name

 -----------------------------------------------------------------------*/
static int mom_set_attribute_program
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
/*  Variable Declarations */
    tag_t       prg = NULL_TAG;
    UF_ATTR_value_t value;
    const char * machine_time , * machine_name ;
    const char * output_file_basename , * output_file_suffix ;
    const char * group_name , * operation_name ;
    char  group_up[MAX_ENTITY_NAME_SIZE+1] ;
    char  file_up[UF_ATTR_MAX_STRING_LEN+1], time_up[UF_ATTR_MAX_STRING_LEN+1], machine_up[UF_ATTR_MAX_STRING_LEN+1];
    int   error = 0 ;
    char title[][UF_ATTR_MAX_TITLE_LEN+1]={
      "OPU_FILE_UP_NAME",
      "OPU_FILE_UP_CREATE",
      "OPU_FILE_UP_TIME",
      "OPU_FILE_UP_MACHINE",
      NULL
    };

    int i ;
    int   count_p = 0 , obj_count = 0;
    tag_t setup_tag, prog_grp;
    tag_t *prog_list = NULL ;
    char  prog_name[MAX_ENTITY_NAME_SIZE+1];
//    int type, subtype ;
    tag_t *tags = NULL ;

    UF_MOM_id_t     mom_id = ( UF_MOM_id_t ) client_data ;

    group_up[0]='\0';
    file_up[0]='\0';
    time_up[0]='\0';
    machine_up[0]='\0';

    prg=null_tag;

    /*   */
    /* MOM group_name file_name time machine */
    if (argc>=5) {

      sprintf(group_up,"%s", argv[1]);
      sprintf(file_up,"%s", argv[2]);
      sprintf(time_up,"%s", argv[3]);
      sprintf(machine_up,"%s", argv[4]);

    } else {

try
{
      error = UF_MOM_ask_string ( mom_id , "group_name" , &group_name ) ;
      if ( error ) return (TCL_OK);
      error = UF_MOM_ask_string ( mom_id , "operation_name" , &operation_name ) ;
      if ( error ) return (TCL_OK);
      error = UF_MOM_ask_string ( mom_id , "output_file_basename" , &output_file_basename ) ;
      if ( error ) return (TCL_OK);
      error = UF_MOM_ask_string ( mom_id , "output_file_suffix" , &output_file_suffix ) ;
      if ( error ) return (TCL_OK);
      sprintf(file_up,"%s.%s", output_file_basename , output_file_suffix);

      error = UF_MOM_ask_string ( mom_id , "machine_time" , &machine_time ) ;
      if ( error ) return (TCL_OK);
      sprintf(time_up,"%s", machine_time);

      error = UF_MOM_ask_string ( mom_id , "machine_name" , &machine_name ) ;
      sprintf(machine_up,"%s", machine_name);
}
catch (...)
{
   return (TCL_OK); // code to handle any exception
}

       /*  
        (     -  */
       /* 1*/
       //     -
       UF_CALL( UF_UI_ONT_ask_selected_nodes(&obj_count, &tags) );
       if (obj_count) {
         for(i=0;i<obj_count;i++)
         {
            prg = tags[i]; //  
            prog_name[0]='\0';
            UF_OBJ_ask_name(prg, prog_name);//   

            /* type =               subtype =
            //     =121              160
            //      =100              110 */
            //UF_CALL( UF_OBJ_ask_type_and_subtype (prg, &type, &subtype ) );

            /*  Assign a string attribute to the object */
                value.type = UF_ATTR_string;
                value.value.string = file_up;
                UF_CALL(UF_ATTR_assign(prg, title[0], value));

            /*  Assign a date and time attribute to the  */
                value.type = UF_ATTR_time;
            /* Init to current date and time */
                UF_CALL(uc4583("", "", value.value.time));
                UF_CALL(UF_ATTR_assign(prg, title[1], value));

            /*  Assign a string attribute to the object */
                value.type = UF_ATTR_string;
                value.value.string = time_up;
                UF_CALL(UF_ATTR_assign(prg, title[2], value));

                /*  Assign a string attribute to the object */
                value.type = UF_ATTR_string;
                value.value.string = machine_up;
                UF_CALL(UF_ATTR_assign(prg, title[3], value));

         }
         UF_free(tags);

         return (TCL_OK);
       }

    }


    /* 2*/
    if (prg==NULL_TAG)
    {
      UF_CALL( UF_SETUP_ask_setup(&setup_tag) );
      UF_CALL( UF_SETUP_ask_program_root(setup_tag, &prog_grp) );
      UF_CALL( UF_NCGROUP_ask_member_list(prog_grp, &count_p, &prog_list) );
      prg=NULL_TAG;
      for(i = 0; i < count_p; i++)
      {
        prog_name[0]='\0';
        UF_OBJ_ask_name(prog_list[i], prog_name);
        if (group_up==NULL) break;
        if (group_up[0]=='\0') break;
        if (strcmp(group_up, prog_name)==0) { prg=prog_list[i]; break ; }
      }
      if (count_p) UF_free(prog_list);
    }

    /* 3*/
    if (prg==NULL_TAG)
    {
      // -         group_up
      error = UF_OBJ_cycle_by_name(group_up,&prg);
      if ( error ) return (TCL_OK);
    }

    if (prg==NULL_TAG) return (TCL_OK);

/*  Assign a string attribute to the object */
    value.type = UF_ATTR_string;
    value.value.string = file_up;
    UF_CALL(UF_ATTR_assign(prg, title[0], value));

/*  Assign a date and time attribute to the  */
    value.type = UF_ATTR_time;
/* Init to current date and time */
    UF_CALL(uc4583("", "", value.value.time));
    UF_CALL(UF_ATTR_assign(prg, title[1], value));

/*  Assign a string attribute to the object */
    value.type = UF_ATTR_string;
    value.value.string = time_up;
    UF_CALL(UF_ATTR_assign(prg, title[2], value));

    /*  Assign a string attribute to the object */
    value.type = UF_ATTR_string;
    value.value.string = machine_up;
    UF_CALL(UF_ATTR_assign(prg, title[3], value));

    return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function :  UGMGR
 *              Part Number,Revision,File Type,File Name,Part Name,Part Desc
 * ask_part_number  ask_part_revision  ask_part_file_type  ask_part_file_name  ask_part_name ask_part_desc

   set name_part [ MOM_ask_part_attr ]

      UGMGR  mom 
     (     global ....    )

       = {part_file_name}_{part_revision}

  :
     name_part=T_T7.92.0562.078.000.73_raz_cam2_A
           mom_ask_part_number      =T_T7.92.0562.078.000.73_raz
           mom_ask_part_revision    =A
           mom_ask_part_file_type   =manifestation
           mom_ask_part_file_name   =T_T7.92.0562.078.000.73_raz_cam2
           mom_ask_part_name        =__
           mom_ask_part_desc        =/ RRJ "..".148;.154;-103.97-006-2005

 -----------------------------------------------------------------------*/
static int mom_ask_part_attr
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  char        set_part_name[MAX_FSPEC_SIZE+UF_UGMGR_FNAME_SIZE+UF_UGMGR_PARTREV_SIZE+1]="";
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;

  UF_MOM_id_t      mom_id = ( UF_MOM_id_t ) client_data ;
  tag_t            work_part_tag;
  char             work_part_name[MAX_FSPEC_SIZE+1]="";
  UF_UGMGR_tag_t   db_tag;
  logical          is_active;
  char             part_number[UF_UGMGR_PARTNO_SIZE+1]="";
  char             part_revision[UF_UGMGR_PARTREV_SIZE+1]="";
  char             part_file_type[UF_UGMGR_FTYPE_SIZE+1]="";
  char             part_file_name[UF_UGMGR_FNAME_SIZE+1]="";
  char             part_name[UF_UGMGR_NAME_SIZE+1]="";
  char             part_desc[UF_UGMGR_DESC_SIZE+1]="";
  char            *p_part_desc;
  char *ptr;

  intrp->result="";
  intrp->errorLine=TCL_OK;

 UF_CALL( UF_is_ugmanager_active ( &is_active ) );

 if (is_active==FALSE)  work_part_tag=UF_PART_ask_display_part();
  else   work_part_tag = UF_ASSEM_ask_work_part();

 if (work_part_tag==null_tag) return (TCL_OK);

 UF_CALL( UF_PART_ask_part_name( work_part_tag, work_part_name ) );

 if (is_active==TRUE)
 {
  UF_print_syslog(" mom_ask_part_attr - Encoding UG/Manager part filename...\n",FALSE);
  UF_CALL( UF_UGMGR_decode_part_filename( work_part_name,  part_number,  part_revision,   part_file_type,   part_file_name ) );
 }

 UF_MOM_set_string ( mom_id , "ask_part_number",part_number ) ;
 UF_MOM_set_string ( mom_id , "ask_part_revision",part_revision ) ;
 UF_MOM_set_string ( mom_id , "ask_part_file_type",part_file_type ) ;
 UF_MOM_set_string ( mom_id , "ask_part_file_name",part_file_name ) ;

 if (is_active==FALSE)
 {
   part_desc[0]='\0'; p_part_desc=part_desc;
   UF_CALL( UF_PART_ask_description(work_part_tag,&p_part_desc) );
 } else {
  UF_CALL( UF_UGMGR_ask_part_tag( part_number, &db_tag ) );
  UF_CALL( UF_UGMGR_ask_part_name_desc( db_tag, part_name, part_desc ) );
 }

 UF_MOM_set_string ( mom_id , "ask_part_name",part_name ) ;
 UF_MOM_set_string ( mom_id , "ask_part_desc",part_desc ) ;

 set_part_name[0]='\0';
 if (is_active==FALSE)
 {
   ptr = strrchr(work_part_name, '\\');
   if (ptr) sprintf(set_part_name,"%s",ptr+1);
   else  sprintf(set_part_name,"%s",work_part_name);
 } else {
   sprintf(set_part_name,"%s_%s",part_file_name,part_revision);
 }

// if (p_part_desc!=NULL) UF_free(p_part_desc);

  intrp->result=set_part_name;
  intrp->errorLine=TCL_OK;

 return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : mom_call_grip
 *                     ?FileName ?param1 ?param2 ?param3 ... ?param30
 *
 -----------------------------------------------------------------------*/
static int mom_call_grip
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  int          i , cnt ;
  char         str[256] ;
  double       user_response = 0;
  int          status = -1;
  char         grip_exe_path[256]="";
  int          grip_arg_count = 30+1;
  UF_args_t    grip_arg_list[30];

  intrp->result="";
  intrp->errorLine=TCL_OK;

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<=1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }

  sprintf(grip_exe_path,"%s",argv[1]);

  /* check file exist */
  UF_CFI_ask_file_exist (grip_exe_path, &status );
  if (status) {
    str[0]='\0'; sprintf(str," UF_CFI_ask_file_exist-File %s not exist\n",grip_exe_path);
    UF_print_syslog(str,FALSE);
    return (TCL_OK);
  }

  cnt=0;
  /*   */
  /* Define the argument list for UG/Open API calling GRIP */
  for (i=2;i<argc;i++) {
     if (i>grip_arg_count) break ;

     grip_arg_list[cnt].type    = UF_TYPE_CHAR ;
     grip_arg_list[cnt].length  = 0 ;
     grip_arg_list[cnt].address = argv[i] ;

     cnt++;
  }

     grip_arg_list[cnt].type    = UF_TYPE_DOUBLE;
     grip_arg_list[cnt].length  = 0;
     grip_arg_list[cnt].address = &user_response;

 /* Execute the GRIP program */
  status = UF_CALL(  UF_call_grip (grip_exe_path, cnt+1, grip_arg_list) );

  str[0]='\0'; sprintf(str," UF_call_grip=%d user_response=%lf\n",status,user_response);
  UF_print_syslog(str,FALSE);

  str[0]='\0'; sprintf(str,"%lf",user_response);

  intrp->result=str;
  intrp->errorLine=TCL_OK;

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : mom_uf_param_ask_value
 *                                     ?operation_name ?param
 *
 *     .
 -----------------------------------------------------------------------*/
static int mom_uf_param_ask_value
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  char         str[256] ;
  int          ecode = 0 ;
  char         operation_name[UF_OPER_OPNAME_LEN+1]="" ;
  int          param = 0 ;
  tag_t        prg   = NULL_TAG;
  int          type ,subtype ;
  tag_t        prt_tag  = NULL_TAG;

  double       dV = 0;
  int          iV = 0;
  logical      lV ;
  tag_t        tagV ;
  char         cV[UF_PARAM_MAX_STRING_LEN+1] ;

  intrp->result="";
  intrp->errorLine=TCL_OK;

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }

  sprintf(operation_name,"%.30s",argv[1]);
  param = atoi (argv[2]) ;

  if ( param<=0 ) { return (TCL_OK); }

  UF_CALL( UF_OBJ_cycle_by_name( operation_name , &prg ) ) ;
/*
  prt_tag = UF_PART_ask_display_part()
  if  (prt_tag != null_tag) {
    UF_OBJ_cycle_by_name_and_type(prt_tag,operation_name,100,TRUE,&prg );
  }
*/
  if (prg==NULL_TAG) return (TCL_OK);

  // 100-; 121-
  UF_CALL( UF_OBJ_ask_type_and_subtype( prg,&type ,&subtype ) );
  if (type!=100) {
    ; //return (TCL_OK);
  }

  ecode=UF_CALL( UF_PARAM_ask_int_value(prg,param,&iV) ) ;
  if (ecode==0) {
    str[0]='\0'; sprintf(str,"%ld",iV);
    UF_print_syslog(str,FALSE);
    intrp->result=str;
    return (TCL_OK);
  }

  ecode=UF_CALL( UF_PARAM_ask_double_value(prg,param,&dV) ) ;
  if (ecode==0) {
    str[0]='\0'; sprintf(str,"%lf",dV);
    UF_print_syslog(str,FALSE);
    intrp->result=str;
    return (TCL_OK);
  }

  ecode=UF_CALL( UF_PARAM_ask_logical_value(prg,param,&lV) ) ;
  if (ecode==0) {
    str[0]='\0'; sprintf(str,"%d",(int)lV);
    UF_print_syslog(str,FALSE);
    intrp->result=str;
    return (TCL_OK);
  }

  ecode=UF_CALL( UF_PARAM_ask_tag_value(prg,param,&tagV) ) ;
  if (ecode==0) {
    str[0]='\0'; sprintf(str,"%o",tagV);
    UF_print_syslog(str,FALSE);
    intrp->result=str;
    return (TCL_OK);
  }

  ecode=UF_CALL( UF_PARAM_ask_str_value(prg,param,cV) ) ;
  if (ecode==0) {
    str[0]='\0'; sprintf(str,"%s",cV);
    UF_print_syslog(str,FALSE);
    intrp->result=str;
    return (TCL_OK);
  }

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : IniRead
 *                     FileName section key
 *
 [section]
key=string
      .
      .
 if section==NULL - > return all section names in the specified file to the supplied buffer
 If key is NULL, the function copies all key names in the specified section to the supplied buffer.

            ini-.
       -  

   :
        key,   
....
 [section]
key=string
....
  set string [ MOM_get_ini_string  "C:\\Windows\\win.ini" "section" "key" ]

 -----------------------------------------------------------------------*/
/*DWORD GetPrivateProfileString(
    LPCTSTR lpAppName,  // points to section name
    LPCTSTR lpKeyName,  // points to key name
    LPCTSTR lpDefault,  // points to default string
    LPTSTR lpReturnedString,  // points to destination buffer
    DWORD nSize,  // size of destination buffer
    LPCTSTR lpFileName  // points to initialization filename
   );
*/

static int mom_IniRead
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  const int Size = 256 ;
  char str[Size];
  int  res = 0;

  char lpAppName[Size]; // points to section name
  char lpKeyName[Size]; // points to key name
  char lpDefault[Size]="";  // points to default string
  char lpReturnedString[Size];  // points to destination buffer
  int  nSize=Size;  // size of destination buffer
  char lpFileName[Size];

  lpReturnedString[0]='\0'; sprintf(lpReturnedString,"%s","");
  intrp->result=lpReturnedString;
  intrp->errorLine=TCL_OK;

  /*   */
  if (argc<4) {
    UF_print_syslog(" IniRead-wrong number arguments\n",FALSE);
    return (TCL_OK);
  }
  lpFileName[0]='\0';sprintf(lpFileName,"%s",argv[1]);
  lpAppName[0]='\0';sprintf(lpAppName,"%s",argv[2]);
  lpKeyName[0]='\0';sprintf(lpKeyName,"%s",argv[3]);

/*  *.ini  */
 HANDLE hFile=CreateFile(lpFileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
 res=GetLastError();
 if(hFile==INVALID_HANDLE_VALUE) {
  str[0]='\0'; sprintf(str," Can't find or read file =%s\n",lpFileName);
  UF_print_syslog(str,FALSE);
  return (TCL_OK);
 }
 CloseHandle(hFile);

 try
 {
   res=GetPrivateProfileString(lpAppName,lpKeyName,lpDefault,lpReturnedString,nSize,lpFileName);
 } catch (...){
    UF_print_syslog("GetPrivateProfileString - Can't open and read *.ini file!\n",FALSE);
 }

  intrp->result=lpReturnedString;
  intrp->errorLine=TCL_OK;

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : mom_remove_file_ex
                             ?FileName
 *
 -----------------------------------------------------------------------*/
static int mom_remove_ex
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  int i ;
  char result[256];

  i = 1 ;
  result[0]='\0';sprintf(result,"%d",i);
  intrp->result=result;
  intrp->errorLine=TCL_OK;

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<=1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }

  /* delete the file */
  i=remove(argv[1]);

  if (i!=0) {
    if (errno==ENOENT) { i=0 ; } // No such file or directory
    else {
     result[0]='\0';sprintf(result,"Don't Removed %s , code error=%d\n",argv[1],i);
     UF_print_syslog(result,FALSE);
    }
  }

  result[0]='\0';sprintf(result,"%d",i);
  intrp->result=result;

  return ( TCL_OK ) ;
}

/*-----------------------------------------------------------------------
 * Function : mom_deletefile
                             ?FileName
 *
 -----------------------------------------------------------------------*/
static int mom_deletefile
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  //LPVOID lpMsgBuf;
  char lpFileName[256] ;
  char str[256];
  int  res ;

  intrp->result="1";
  intrp->errorLine=TCL_OK;

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<=1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }

  sprintf(lpFileName,"%s",argv[1]);

/*  *.*  */
 HANDLE hFile=CreateFile(lpFileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
 res=GetLastError();
 if(hFile==INVALID_HANDLE_VALUE) {
  str[0]='\0'; sprintf(str," Can't find or read file =%s\n",lpFileName);
  UF_print_syslog(str,FALSE);
  intrp->result="0";
  return (TCL_OK);
 }
 CloseHandle(hFile);

  if (!DeleteFile((LPCTSTR)lpFileName)) {

   //FormatMessage(
   //    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
   //    NULL,
   //    GetLastError(),
   //    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
   //    (LPTSTR) &lpMsgBuf,
   //    0,
   //    NULL
   //);
   //
   ////intrp->result=(char*)lpMsgBuf;
   //printf("\n%s",(char*)lpMsgBuf);
   //
   //// Free the buffer.
   //LocalFree( lpMsgBuf );

  } else {

   intrp->result="0";

  }
  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : mom_createfile
                             ?FileName
 *
 -----------------------------------------------------------------------*/
static int mom_createfile
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  Tcl_Interp  *intrp = ( Tcl_Interp *) interp ;
  char lpFileName[256] ;
  int  res ;
  char str[256];

  intrp->result="1";
  intrp->errorLine=TCL_OK;

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<=1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }

  sprintf(lpFileName,"%s",argv[1]);

/*  *  */
 HANDLE hFile=CreateFile(lpFileName,
                         GENERIC_WRITE, // GENERIC_READ
                         FILE_SHARE_WRITE,
                         NULL,
                         CREATE_ALWAYS,
                         FILE_ATTRIBUTE_NORMAL,
                         NULL);
 res=GetLastError();
 if(hFile==INVALID_HANDLE_VALUE) {
  str[0]='\0'; sprintf(str," Can't Find/Create or read/write file =%s\n",lpFileName);
  UF_print_syslog(str,FALSE);
  return (TCL_OK);
 }
 CloseHandle(hFile);

 intrp->result="0";

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : RunExecute
 *
 *
    MOM_run_exec ?"exe ?param1 ?param2 .."
     
    MOM_run_exec ?exe ?param1 ?param2 ..

       exe-     
       -  
 -----------------------------------------------------------------------*/
static int mom_RunExecute
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  const int Size = 1024 ;
  int  i ;
  int  l ;
  char str[Size] ;

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<=1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }

  ZeroMemory( &str, sizeof(str) );
  /*   */
  str[0]='\0';
  sprintf(str,"\"");
  for (i=1;i<argc;i++) {
     l=(int)strlen(str)+(int)strlen(argv[i]);
     if (l>=Size) break ;
     strcat(str,argv[i]);
     strcat(str,"\" ");
  }

    STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

    // Start the child process.
    if( !CreateProcess( NULL, // No module name (use command line).
        str,       // Command line.
        NULL,             // Process handle not inheritable.
        NULL,             // Thread handle not inheritable.
        FALSE,            // Set handle inheritance to FALSE.
        0,                // No creation flags.
        NULL,             // Use parent's environment block.
        NULL,             // Use parent's starting directory.
        &si,              // Pointer to STARTUPINFO structure.
        &pi )             // Pointer to PROCESS_INFORMATION structure.
    )
    {
      UF_print_syslog(" CreateProcess don't Start the child process \n",FALSE);
    }

/*try
{
     //   
     if (WinExec(str,1) < 32) {
        str[0]='\0';  sprintf(str, "WinExec failed; error code = %d\n", i);
        UF_print_syslog(str,FALSE);
     }
     //   -
     _spawnl (_P_NOWAIT,str,NULL,NULL,NULL);
}
catch (...)
{
   // code to handle any exception
   UF_print_syslog(" Process don't Start the child process \n",FALSE);
}
*/

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : ShellExecute
 *
 *
      exe- (  )     
       -  

 -----------------------------------------------------------------------*/
static int mom_ShellExecute
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  char *param=NULL;
  char str[256];

  if (argc<2) { return (TCL_OK); }
  if (strlen(argv[1])<=1) { return (TCL_OK); }
  if (argv[1]==NULL) { return (TCL_OK); }
  if (argc>=3) {
    if (strlen(argv[2])>0) {
      str[0]='\0';param=str;
      strcpy(param,argv[2]);
    }
  }

  try
  {
   ShellExecute( NULL, NULL ,argv[1],param,NULL,SW_SHOW );
  } catch (...){
    UF_print_syslog(" ShellExecute execute with error \n",FALSE);
  }

  return ( TCL_OK ) ;
}


/*-----------------------------------------------------------------------
 * Function : AboutStub
 *
 *
 -----------------------------------------------------------------------*/
static int mom_AboutStub
(
  void *client_data,    /* This is the mom object */
  void *interp,         /* Pointer to the TCL interpreter */
  int  argc,            /* Count of the numnber of arguments */
  char **argv           /* Array of arguments where the
                           argv[0] - The TCL extension */
)
{
  char str[133];

  str[0]='\0'; sprintf(str, "4 & Rodionova,@2002 years\n continue in 2003 years");

  try
  {
    ShellAbout (NULL,"MOM Stub Extended for UniGraphics",str,NULL);
  } catch (...) {
      UF_print_syslog(" ShellAbout error process \n",FALSE);
      uc1601 (str, TRUE );
  }

  return ( TCL_OK ) ;
}

/****************************************************************************/
